#include "Geometry.h"

using namespace std;


// Geometrie dans le cas ou espace de Position et espace de traits Phenotypiques sont dans paves.


//construction standard [0,1]*[0,1]
Geometry::Geometry()
{
    vector<double> mins(1,0);
    vector<double> maxs(1,1);
    m_minPositions = mins;
    m_maxPositions = maxs;
    m_minPhenotypes = mins;
    m_maxPhenotypes = maxs;
}


//constructeur si les parametres sont donnes sous forme de double et non vector<double>
Geometry::Geometry(double const& minPosition, double const& maxPosition,
                 double const& minPhenotype, double const& maxPhenotype)
{
    vector<double> vecminposition(1,minPosition);
    vector<double> vecmaxposition(1,maxPosition);
    vector<double> vecminphenotype(1,minPhenotype);
    vector<double> vecmaxphenotype(1,maxPhenotype);
    m_minPositions = vecminposition;
    m_maxPositions = vecmaxposition;
    m_minPhenotypes = vecminphenotype;
    m_maxPhenotypes = vecmaxphenotype;
}


//construction en suivant les parametres donnes
Geometry::Geometry(double const& minPosition, double const& maxPosition,
                 std::vector<double> const& minPhenotypes, std::vector<double> const& maxPhenotypes):
                 m_minPhenotypes(minPhenotypes), m_maxPhenotypes(maxPhenotypes)
{
    vector<double> mins(1,minPosition);
    vector<double> maxs(1,maxPosition);
    m_minPositions = mins;
    m_maxPositions = maxs;
}


//destructeur
Geometry::~Geometry()
{
}


//verifie si la position spatiale est bien a l'interieur du domaine
bool Geometry::isInsidePosition(double const& positionTested) const
{
    return (positionTested > m_minPositions[0]) && (positionTested < m_maxPositions[0]);
}


//renvoie la distance entre la position tested et le debut de l'intervalle
double Geometry::distanceMinPosition(double const& positionTested, int const& indexDimension) const
{
    if (isInsidePosition(positionTested))
    {
        return positionTested - m_minPositions[indexDimension];
    }
    else 
    {
        cout << "segmentation error in distanceMinPosition" << endl;
        return -1;
    }
}


//donne la taille de l'intervalle de Position de la n-eme dimension
double Geometry::lengthIntervalPosition(int const& indexDimension) const
{
    return m_maxPositions[indexDimension]-m_minPositions[indexDimension];
}


//verifie si le trait est bien a l'interieur du domaine phenotypique
bool Geometry::isInsidePhenotype(double const& phenotypeTested, int const& index) const
{
    //bool test(true);
    //int nbrPhenotypeTraits(m_minPhenotypes.size());
    //for (int k(0); k<nbrPhenotypeTraits; k++)
    //{
    //    if ((phenotypeTested[k] < m_minPhenotypes[k]) || (phenotypeTested[k] > m_maxPhenotypes[k]))
    //        test = false;
    //}
    return ((phenotypeTested > m_minPhenotypes[index]) &&
             (phenotypeTested < m_maxPhenotypes[index]));
}


//renvoie la distance entre le trait Phenotypique tested et le debut de l'intervalle
double Geometry::distanceMinPhenotype(vector<double> const& phenotypeTested, int const& indexDimension) const
{
    //if (isInsidePhenotype(phenotypeTested))
        return phenotypeTested[indexDimension] - m_minPositions[indexDimension];
    //else return -1;
}


//donne la taille de l'intervalle de Position de la n-eme dimension
double Geometry::lengthIntervalPhenotype(int const& indexDimension) const
{
    return m_maxPhenotypes[indexDimension]-m_minPhenotypes[indexDimension];
}


//calcul la diffusion pour un brownien partant de initialPosition pendant un temps tInterval,
//dans le segment minPosition, maxPosition
double Geometry::diffusionBrownian(double const& tInterval, double const& coefDiffusion,
                                   double const& initialPosition) const
{
    if (tInterval < 0.0000001)
    {
        return initialPosition;
    }
    else
    {
        double gaussRand( sqrt( -2.0*log( ((double)rand()+1.0) / ((double)RAND_MAX+2.0)) ) * cos( 6.2852*(double)rand() / ((double)RAND_MAX+1.0) ) );
        assert(abs(gaussRand < 1000));
        double positionTemp( initialPosition + sqrt(2.0 * coefDiffusion * tInterval) * gaussRand);
        if ( !isInsidePosition(positionTemp) )
        {
            double lengthInterval(m_maxPositions[0] - m_minPositions[0]);
            assert(lengthInterval != 0);
            int nbrInterval((positionTemp - m_minPositions[0])/lengthInterval);
            if (positionTemp < m_minPositions[0])
                {
                nbrInterval -= 1;
                }
            positionTemp -= lengthInterval*nbrInterval;
            if ( abs(nbrInterval%2) == 1)
            {
                positionTemp = m_maxPositions[0] + m_minPositions[0] - positionTemp;
            }
        }
        assert(isInsidePosition(positionTemp));
        return positionTemp;
    }
}

