#ifndef GEOMETRY_H_INCLUDED
#define GEOMETRY_H_INCLUDED

#include <iostream>
#include <cmath>
#include <algorithm>
#include <stdlib.h>     /* srand, rand */
#include <vector>
#include <assert.h>

// Geometrie dans le cas ou espace de Position et espace de traits Phenotypiques sont dans paves.

class Geometry{

    private:
        std::vector<double> m_minPositions;
        std::vector<double> m_maxPositions;
        std::vector<double> m_minPhenotypes;
        std::vector<double> m_maxPhenotypes;

    public:
        //constructeur de base: espace = [0,1]*[0,1] (Positions*Traits)
        Geometry();
        Geometry(double const& minPosition, double const& maxPosition,
                 double const& minPhenotype, double const& maxPhenotype);
        Geometry(double const& minPosition, double const& maxPosition,
                 std::vector<double> const& minPhenotypes, std::vector<double> const& maxPhenotypes);
        ~Geometry();
        bool isInsidePosition(double const& positionTested) const;
        double distanceMinPosition(double const& positionTested, int const& indexDimension) const;
        double lengthIntervalPosition(int const& indexDimension) const;
        bool isInsidePhenotype(double const& phenotypeTested, int const& index) const;
        double distanceMinPhenotype(std::vector<double> const& phenotypeTested, int const& indexDimension) const;
        double lengthIntervalPhenotype(int const& indexDimension) const;
        double diffusionBrownian(double const& tInterval, double const& coefDiffusion,
                                 double const& initialPosition) const;




};


#endif // GEOMETRY_H_INCLUDED
