#ifndef POPULATION_H_INCLUDED
#define POPULATION_H_INCLUDED


#include <iostream>
#include <fstream>     /* affichage vers fichier .txt */
#include <cmath>        /* outils mathematiques */
#include <algorithm>
#include <stdlib.h>     /* srand, rand */
#include <vector>
#include <assert.h>
#include "Entity.h"

class Population
{

    private :
        std::vector<std::vector<Entity> > m_tableEntities;


    public :
        //constructeur destructeur
        Population();
        Population(Entity const& firstEntity);
        Population(Entity const& firstEntity, int const& nbrEntities);
        ~Population();

        //nous informe sur la population
        int nbrEntities() const;
        int nbrEntitiesInType(int const& typeEntity) const;
        int nbrTypes() const;
        bool isAlive() const;

        //taux de mort
        double killRateCompetition(double const& timeNow, int const& indexEntity1,
                                   int const& typeEntity1, int const& indexEntity2,
                                   int const& typeEntity2);
        double killRateCompetitionMut(double const& timeNow, int const& indexEntity1,
                                   int const& typeEntity1, int const& indexEntity2,
                                   int const& typeEntity2);
        double killRateNatural(double const& timeNow, int const& indexEntity,
                                   int const& typeEntity);
        //suppression de l'individu n°nKilled
        void killEntity(int const& nKilled, int const& typeKilled);
        //taux de naissance
        double birthRateNatural(double const& timeNow, int const& indexEntity,
                                   int const& typeEntity);
        double birthRateInteraction(double const& timeNow, int const& indexEntity1,
                                   int const& typeEntity1, int const& indexEntity2,
                                   int const& typeEntity2);
        // verifie si l'entite a ajouter fait partie d'une espece connue ou non (quitte a l'ajouter)
        bool knownSpecies(int const& typeEntity) const;
        //ajout de l'individu oneAdded
        void addEntity(Entity const& oneAdded);
        //supprime tous les individus de la population
        void empty();
        //met a jour les positions de tous les individus a un instant donne et teste si un 
        //individu a atteint la position limite
        bool update_test(double const& timeNow, double const& testedPosition);
        

        //ecriture de la population dans un fichier txt, une ligne par individu
        void writePopulation(std::string nameFileOut, double const& timeNow);
        //ecriture des traits+nbr d'ind par traits
        void writeTraits(std::string nameFileOut, double const& timeNow);


        //nous indique par les boolean isDead et isBirth s'il y a naissance ou mort a partir du calcul de l'alea theta pour une espece donnee
        void chooseEvent(double const& timeNow, double const& birthRate, double const& deathRate,
                         double const& interRate, int const& typePartner, double const& compIntraRate,
                         const double &thetaInitial, int const& indexEndangered,
                         int const& typeEndangered, bool &isDead, bool &isBirth);
        //un evenement: naissance, mort ou rien, a partir de chooseEvent
        void oneStep(double const& timeNow, double const& birthRate, const double &deathRate,
                     const double &interRate, int const& typePartner, double const& compRate,
                     double const& thetaInitial, const int &indexEndangered, const int &typeEndangered);
        //un evenement: naissance, mort ou rien, a partir de chooseEvent
        //+ indique s'il y a eu apparition un nouveau trait
        void oneStep_isMutation(double const& timeNow, double const& birthRate, const double &deathRate,
                     const double &interRate, int const& typePartner, double const& compRate,
                     double const& thetaInitial, const int &indexEndangered, const int &typeEndangered,
                     bool &isMutation);

        //un evenement pour une pop constitue de 2 especes
        void oneStepTwoSpecies(double& timeNow, double const& maxBirthRatebees, double const& maxDeathRatebees, 
                               double const& maxInterRatebees, double const& maxCompRatebees, 
                               double const& maxBirthRateplants, double const& maxDeathRateplants, 
                               double const& maxInterRateplants, double const& maxCompRateplants);
        //un evenement pour une pop constitue de 1 espece
        //u+ indique dans le boolean si il y a eu apparition un nouveau trait
        void oneStepOneSpecies(double& timeNow, double const& maxBirthRatebees, double const& maxDeathRatebees, 
                                   double const& maxCompRatebees, bool &isMutation);



};


#endif // POPULATION_H_INCLUDED
