#include "rateMove.h"
#include <iostream>
#include <cmath>        /* outils mathematiques */
#include <algorithm>
#include <stdlib.h>     /* srand, rand */
#include <vector>

using namespace std;


/* dans ce cas, l'espece est identifiee par l'indice dans parameters[0]
 = 1 --> abeilles, donc bougent tout le temps
 = 2 --> plantes, bougent uniquement a la naissance */

/*Attention, les fonctions definies ici doivent avoir des valeurs entre 0 et 1, excepte coefDiffusion et les shift.
*/

/* Rate */


// calcul le taux de naissance naturel a partir des traits
double birthRate(double const& traitPosition, vector<double> const& traitPhenotype,
                 vector<double> const& parameters)
{
    int typeSpecies((int) parameters[0]);
    double rate(1);
    double difference(traitPosition-traitPhenotype[0]);

    switch (typeSpecies)
    {
    case 0: //cas abeilles
        {
        return rate;
        }
    break;

    case 1: //cas plantes
        rate=1.-40.*pow(difference,2);
        if (rate < 0) rate = 0.;
        return rate;
    break;

    default: //cas rate!
        cout << "No such species!";
    break;
    }
}


//calcul le noyau d'interaction pour un taux positif
double positiveInteractionKernel(double const& traitPosition1, vector<double> const& traitPhenotype1,
                                 vector<double> const& parameters1, double const& traitPosition2,
                                 vector<double> const& traitPhenotype2, vector<double> const& parameters2)
{
    int typeSpecies1((int) parameters1[0]);
    int typeSpecies2((int) parameters2[0]);
    double kernel(0);

    switch (typeSpecies1)
    {
    case 0: //cas abeille
        {
        switch (typeSpecies2)
        {
        case 0: //cas abeille
            {
            return 0;
            }

        case 1: //cas plante
            {
            if (abs(traitPosition1-traitPosition2) <= parameters1[5]
                && abs(traitPhenotype1[0]-traitPhenotype2[0]) <= parameters1[9])
                kernel = 1;
            return kernel;
            }

        default:
            cout << "No such species!";
        break;
        }
        }
    break;

    case 1: //cas plante
        {
        switch (typeSpecies2)
        {
        case 0: //cas abeille
            {
            if (abs(traitPosition1-traitPosition2) <= parameters1[5]
                && abs(traitPhenotype1[0]-traitPhenotype2[0]) <= parameters1[9])
              kernel = 1;
            return kernel;
            }

        case 1: //cas plante
            {
            return 0;
            }

        default:
            cout << "No such species!";
        break;
        }
        }
    break;

    default: //cas rate!
        cout << "No such species!";
    break;
    }

}



//calcul le taux de mort naturel a partir des traits
double deathRate(double const& traitPosition, vector<double> const& traitPhenotype,
                 vector<double> const& parameters)
{
    int typeSpecies((int) parameters[0]);

    switch (typeSpecies)
    {
    case 0: //cas abeilles
        return 1;
    break;

    case 1: //cas plantes
        return 1;
    break;

    default: //cas rate!
        cout << "No such species!";
    break;
    }
}


// calcul le noyau de competition (noyau d'interaction negatif),
//associe a un rayon d'interaction "range"
double competitionKernel(double const& traitPosition1, vector<double> const& traitPhenotype1,
                         vector<double> const& parameters1, double const& traitPosition2,
                         vector<double> const& traitPhenotype2, vector<double> const& parameters2)
{
    int typeSpecies1((int) parameters1[0]);
    int typeSpecies2((int) parameters2[0]);
    double kernel(0);

    switch (typeSpecies1)
    {
    case 0: //cas abeille subit la competition
        {
        switch (typeSpecies2)
        {
        case 0: //cas abeille est le competiteur
            {
            if (traitPosition2 <= traitPosition1 + parameters1[3] && 
                traitPosition2 >= traitPosition1 - parameters1[3])
                kernel = 1;
            return kernel;
            }

        case 1: //cas plante est le competiteur
            {
            return 0;
            }

        default:
            cout << "Error in competitionKernel function";
        break;
        }
        }
    break;

    case 1: //cas plante subit la competition
        {
        switch (typeSpecies2)
        {
        case 0: //cas abeille est le competiteur
            {
            return 0;
            }

        case 1: //cas plante est le competiteur
            {
            if (abs(traitPosition1-traitPosition2) <= parameters1[3])
                kernel = 1;
            return kernel;
            }

        default:
            cout << "Error in competitionKernel function";
        break;
        }
        }
    break;

    default: //cas rate!
        cout << "Error in competitionKernel function";
    break;
    }

}


// calcul le noyau de competition (noyau d'interaction negatif),
//associe a un rayon d'interaction "range"
double competitionMutKernel(double const& traitPosition1, vector<double> const& traitPhenotype1,
                         vector<double> const& parameters1, double const& traitPosition2,
                         vector<double> const& traitPhenotype2, vector<double> const& parameters2)
{
    int typeSpecies1((int) parameters1[0]);
    int typeSpecies2((int) parameters2[0]);
    double kernel(0);

    switch (typeSpecies1)
    {
    case 0: //cas abeille subit la competition
        {
        switch (typeSpecies2)
        {
        case 0: //cas abeille est le competiteur
            {
            if (abs(traitPosition1-traitPosition2) <= parameters1[5])
                kernel = 1;
            return kernel;
            }

        case 1: //cas plante est le competiteur
            {
            return kernel;
            }

        default:
            cout << "Error in competitionMutKernel function";
        break;
        }
        }
    break;

    case 1: //cas plante subit la competition
        {
        switch (typeSpecies2)
        {
        case 0: //cas abeille est le competiteur
            {
            return kernel;
            }

        case 1: //cas plante est le competiteur
            {
            if (abs(traitPosition1-traitPosition2) <= parameters1[5])
                kernel = 1;
            return kernel;
            }

        default:
            cout << "Error in competitionMutKernel function";
        break;
        }
        }
    break;

    default: //cas rate!
        cout << "Error in competitionMutKernel function";
    break;
    }

}


// calcul du trait du nouveau mutant
vector<double> mutantTraitPhenotype(vector<double> const& traitPhenotype, double const& traitPosition,
                                    vector<double> const& parameters, Geometry const& geometrySpace)
{
    int sizeTrait(traitPhenotype.size());
    vector<double> newTrait(sizeTrait);

        //premier trait
    double mutant1;
    do
        {
        mutant1 = traitPhenotype[0] + parameters[1] * sqrt(-2.0 * log( (rand()+1.)/((double)RAND_MAX+2.0) ))
        * cos( 6.2852 * rand() /((double)RAND_MAX+1.) );
        }
    while ( !geometrySpace.isInsidePhenotype(mutant1,0) );
    newTrait[0] = mutant1;
    //ici pas d'autres traits!

    int typeSpecies((int) parameters[0]);

    switch (typeSpecies)
    {
    case 0: //cas abeilles
        return newTrait;
    break;

    case 1: //cas plantes
        return newTrait;
    break;

    default: //cas rate!
        cout << "Ne connait pas de telle espece";
    break;
    }

}


/* Move */

//calcul du coefficient de diffusion
double coefDiffusion(double const& traitPosition, vector<double> const& traitPhenotype,
                     vector<double> const& parameters)
{
    int typeSpecies((int) parameters[0]);

    switch (typeSpecies)
    {
    case 0: //cas abeilles
        return parameters[2];
    break;

    case 1: //cas plantes
        return 0.;
    break;

    default: //cas rate!
        cout << "No such species!";
    break;
    }
}

//mouvement pendant un temps timeInterval pendant la vie de l'Entity
double liveShifting(double const& timeInterval, Geometry const& geometrySpace,
                    double const& traitPosition,
                    vector<double> const& traitPhenotype, vector<double> const& parameters)
{
    int typeSpecies((int) parameters[0]);

    switch (typeSpecies)
    {
    case 0: //cas abeilles
        {
        double diffusion(coefDiffusion(traitPosition, traitPhenotype, parameters));
        return geometrySpace.diffusionBrownian(timeInterval, diffusion, traitPosition);
        }
    break;
    case 1: //cas plantes: pas de mouvement!
        return traitPosition;
    break;

    default: //cas rate!
        cout << "No such species!";
    break;
    }
}

//deplacement a la naissance de l'entity
double birthShifting(double const& traitPosition, vector<double> const& traitPhenotype,
                     vector<double> const& parameters, Geometry const& geometrySpace)
{
    int typeSpecies((int) parameters[0]);

    switch (typeSpecies)
    {
    case 0: //cas abeilles: pas de mouvement a la naissance
        return traitPosition;
    break;

    case 1: //cas plantes: dispersion a la naissance!
        {
        double newPosition;
        do
            {
            newPosition = traitPosition + parameters[2] * sqrt(-2.0 * log( (rand()+1.)/((double)RAND_MAX+2.0) ))
            * cos( 6.2852 * rand() /((double)RAND_MAX+1.) );
            }
        while ( !geometrySpace.isInsidePosition(newPosition) );
        return newPosition;
        }
    break;

    default: //cas rate!
        cout << "No such species!";
    break;
    }
}
